#========================================================================================
# package      DE_EPAGES::Design::t::StyleHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the TLE functions of the package
#               DE_EPAGES::Design::API::TLE::StyleHandler
#========================================================================================
package DE_EPAGES::Design::t::StyleHandler;

use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Design::API::TLE::StyleHandler;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::TLE::API::BaseProcessor;
use Test::More;
use strict;

use constant WEBURLUNITYSSL => 'http://www.my-testshop.com';
use constant THEME_SPOTLIGHT => 'epages.spotlight';
use constant VERSION_DEV => 'dev';
use constant STYLE_STYLE => 'style';
use constant STYLE_PATTERN => 'pattern';
use constant THEME_NEUTRAL => 'epages.neutral';
use constant VERSION_2 => '2';

sub test {
    my $self = shift;

    $self->_test_GetUnityThemeStyleURL();

    done_testing();
}

sub _test_GetUnityThemeStyleURL {
    my $self = shift;

    # Build a wrapper function to use the TLE function "GetUnityThemeStyleURL" in this Perl code
    my $Processor = DE_EPAGES::TLE::API::BaseProcessor->new;
    my $StyleHandler = DE_EPAGES::Design::API::TLE::StyleHandler->new();
    $StyleHandler->register($Processor);
    my $GetUnityThemeStyleURL = sub {
        my ($Object, $ChangeAction) = @_;
        $Processor->callBlock('GetUnityThemeStyleURL', sub {[ $Object, $ChangeAction ]});
    };

    # simulate broken Unity connection
    my $Shop = $self->_createBrokenShop();
    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), undef,
        "No UnityThemeStyleURL constructed for Object \"Shop\" and ChangeAction \"PickupBasket\" in case of a broken Unity connection.");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), undef,
        "No UnityThemeStyleURL constructed for Object \"Shop\" and ChangeAction \"SaveCustomerData\" in case of a broken Unity connection.");

    # test theme: epages.spotlight@dev, style: style
    $Shop = $self->_createShop(THEME_SPOTLIGHT, VERSION_DEV, STYLE_STYLE);
    my $UnityThemeStyleURL = $self->_constructUnityThemeStyleURL(THEME_SPOTLIGHT, VERSION_DEV, STYLE_STYLE);

    is($GetUnityThemeStyleURL->($Processor, ''), undef,
        "No UnityThemeStyleURL constructed for Object \"Processor\" and empty ChangeAction.");
    is($GetUnityThemeStyleURL->($Processor, 'PickupBasket'), undef,
        "No UnityThemeStyleURL constructed for Object \"Processor\" and ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), undef,
        "No UnityThemeStyleURL constructed for Object \"Shop\" and ChangeAction \"SaveCustomerData\".");
    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), $UnityThemeStyleURL,
        "Theme \"" . THEME_SPOTLIGHT . "@" . VERSION_DEV . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL constructed properly for ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "@" . VERSION_DEV . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"SaveCustomerData\".");
    is($GetUnityThemeStyleURL->($Shop, ''), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "@" . VERSION_DEV . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"\".");
    is($GetUnityThemeStyleURL->($Shop, undef), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "@" . VERSION_DEV . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for undef ChangeAction.");

    # change style => test theme: epages.spotlight@dev, style: pattern
    $Shop = $self->_createShop(THEME_SPOTLIGHT, VERSION_DEV, STYLE_PATTERN);

    #change the stored UnityThemeStyleURL using the change action PickupBasket
    $UnityThemeStyleURL = $self->_constructUnityThemeStyleURL(THEME_SPOTLIGHT, VERSION_DEV, STYLE_PATTERN);
    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), $UnityThemeStyleURL,
        "Theme \"" . THEME_SPOTLIGHT . "@" . VERSION_DEV . "\", Style \"" . STYLE_PATTERN
        . "\": UnityThemeStyleURL constructed properly for ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "@" . VERSION_DEV . "\", Style \"" . STYLE_PATTERN
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"SaveCustomerData\".");

    # change theme => test theme: epages.neutral@2, style: style
    $Shop = $self->_createShop(THEME_NEUTRAL, VERSION_2, STYLE_STYLE);

    #change the stored UnityThemeStyleURL using the change action PickupBasket
    $UnityThemeStyleURL = $self->_constructUnityThemeStyleURL(THEME_NEUTRAL, VERSION_2, STYLE_STYLE);
    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), $UnityThemeStyleURL,
        "Theme \"" . THEME_NEUTRAL . "@" . VERSION_2 . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL constructed properly for ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), $UnityThemeStyleURL,
       "Theme \"" . THEME_NEUTRAL . "@" . VERSION_2 . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"SaveCustomerData\".");

    return;
}

sub _createShop {
    my $self = shift;
    my ($ThemeName, $ThemeVersion, $StyleName) = @_;

    my $hAttributes = {
        'WebUrlUnitySSL' => WEBURLUNITYSSL,
        'instanceOf' => 'Shop',
        'UnityThemeStylePath' => undef
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValue('getUnityApiShopContent', {
        "theme" => {
            "name" => $ThemeName,
            "version" => $ThemeVersion,
            "themeSettings" => {
                "settings" => { "style" => $StyleName }
            }
        }
    });
    return $MockifyShop->getMockObject();
}

sub _createBrokenShop {
    my $self = shift;
    my ($ThemeName, $ThemeVersion, $StyleName) = @_;

    my $hAttributes = {
        'WebUrlUnitySSL' => WEBURLUNITYSSL,
        'instanceOf' => 'Shop',
        'UnityThemeStylePath' => undef
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValue('getUnityApiShopContent', undef);
    return $MockifyShop->getMockObject();
}

sub _constructUnityThemeStyleURL {
    my $self = shift;
    my ($ThemeName, $ThemeVersion, $StyleName) = @_;
    return WEBURLUNITYSSL . '/themes/' . $ThemeName . '@' . $ThemeVersion
            . '/stylesheets/' . $StyleName . '-checkout.css';
}

__PACKAGE__->runScript();


1;
