#========================================================================================
# package      DE_EPAGES::Mollie::t::unit::PaymentRequest
# base         DE_EPAGES::Database::API::Test
# state        private
# description  API/PaymentRequest.pm unit test
#========================================================================================
package DE_EPAGES::Mollie::t::PaymentRequest;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Mollie::API::MolliePayment;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject;
use Test::MockObject::Extends;
use Test::More;

use constant ORDER_GUID => '00000000-0000-0000-0000-000000000000';

use strict;

## no critic qw(ProhibitLongLines)

#========================================================================================
sub test {
    my $self = shift;
    $self->createPayment_positive_test();
    $self->createPayment_LineItemPayment_parameter_missed_test();
    $self->createPayment_Locale_parameter_missed_test();
    $self->createPayment_PaymentRequest_error_test();
    $self->createPayment_PaymentRequest_empty_test();
    done_testing();
}

#========================================================================================
sub createPayment_positive_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $LineItemPayment = $self->_createLineItemPaymentMollie();
    my $Locale          = $self->_createLocale();
    my $isMobile        = '0';

    my $PaymentRequestOK = 1;
    my $MolliePayment    = $self->_createMolliePayment($PaymentRequestOK);

    $MolliePayment->createPayment( $LineItemPayment, $Locale, $isMobile);

    is(
        $MolliePayment->{'PaymentURL'},
'https://www.mollie.com/paymentscreen/creditcard/testmode/dc0aa798840b6cfacc203277df02174f',
        "$SubTestName - PaymentURL is OK"
    );
    is( $MolliePayment->{'TransactionID'},
        'tr_MU84QbtAms', "$SubTestName - TransactionID is OK" );
}

#========================================================================================
sub createPayment_LineItemPayment_parameter_missed_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $LineItemPayment = undef;
    my $Locale          = $self->_createLocale();
    my $isMobile        = '0';

    my $PaymentRequestOK = 1;
    my $MolliePayment    = $self->_createMolliePayment($PaymentRequestOK);

    throws_ok { $MolliePayment->createPayment( $LineItemPayment, $Locale, $isMobile ) }
    qr/PARAMETER_MISSING/,
      "$SubTestName - Missed LineItemPayment Parameter - Exception is thrown.";

}

#========================================================================================
sub createPayment_Locale_parameter_missed_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $LineItemPayment = $self->_createLineItemPaymentMollie();
    my $Locale          = undef;
    my $isMobile        = '0';

    my $PaymentRequestOK = 1;
    my $MolliePayment    = $self->_createMolliePayment($PaymentRequestOK);

    throws_ok { $MolliePayment->createPayment( $LineItemPayment, $Locale, $isMobile ) }
    qr/PARAMETER_MISSING/,
      "$SubTestName - Missed LineItemPayment Parameter - Exception is thrown.";

}

#========================================================================================
sub createPayment_PaymentRequest_error_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $LineItemPayment = $self->_createLineItemPaymentMollie();
    my $Locale          = $self->_createLocale();
    my $isMobile        = '0';

    my $PaymentRequestOK = 0;
    my $MolliePayment    = $self->_createMolliePayment($PaymentRequestOK);

    throws_ok { $MolliePayment->createPayment( $LineItemPayment, $Locale, $isMobile ) }
    qr/Error Creating the payment request at epagesj/,
      "$SubTestName - Payment Request trhows error - Exception is thrown.";

}

#========================================================================================
sub createPayment_PaymentRequest_empty_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $LineItemPayment = $self->_createLineItemPaymentMollie();
    my $Locale          = $self->_createLocale();
    my $isMobile        = '0';

    my $PaymentRequestOK = 2;
    my $MolliePayment    = $self->_createMolliePayment($PaymentRequestOK);

    throws_ok { $MolliePayment->createPayment( $LineItemPayment, $Locale, $isMobile ) }
    qr/PaymentURL or TransactionID are empty/,
      "$SubTestName - Empty response from EpagesJ - Exception is thrown.";

}

#========================================================================================
sub _createLineItemPaymentMollie {
    my $self = shift;

    my $aParameterList = [
        'Site'   => $self->_createShop(),
        'GUID'   => 'LineItemPaymentMollie',
        'Amount' => '12.25',
        'PaymentMethod' => $self->_createPaymentMethod(),
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Mollie::API::Object::LineItemPaymentMollie',
        $aParameterList );

    $Mockify->addMock(
        'parent',
        sub {
            shift;
            return $self->_createLineItemContainer();
        }
    );

    return $Mockify->getMockObject();
}

#========================================================================================
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes       = {};
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );

    $LineItemContainer->mock(
        'parent',
        sub {
            shift;
            return $self->_createOrder();
        }
    );

    return $LineItemContainer;
}

#========================================================================================
sub _createOrder {
    my $self = shift;

    my $hAttributes = { 'GUID' => ORDER_GUID, 'Alias' => 'Alias' };
    my $Order = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes );

    return $Order;
}

#=======================================================================================
sub _createShop {
    my $self = shift;

    my $ShopObject = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
         {
             'ProtocolAndServerSSL' => 'http://BaseURL',
             'Alias' => 'AliasShop'
         } );

    $ShopObject->mock(
        'id',
        sub {
            shift;
            return 'DemoShop';
        }
    );

    return $ShopObject;
}

#=======================================================================================
sub _createLocale {
    my $self = shift;

    return 'en_US';
}

#=======================================================================================
sub _createMolliePayment {
    my $self = shift;
    my ($PaymentRequestOK) = @_;

    my $aParameterList = [];
    my $Mockify        = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Mollie::API::MolliePayment',
        $aParameterList );

    my $aParameterTypesLog = [ 'string', 'string', 'any' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_LogPayment', 1,
        $aParameterTypesLog );

    my $aParameterTypesGetReturn = [ 'object', 'string', 'string' ];
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_getReturnURL',
'http://my-vbox-10.intern.epages.de:8088/rs/payments/mollie/payment?amount=79.95&orderid=55E96EB0-ACD3-D3C8-D19B-C0A8380A43A1&description=Epages+Mollie+Payment&redirectUrl=http%3A%2F%2F192.168.56.10%2Fepages%2FDemoShop.sf%2F%3FChangeAction%3DPaymentBacklinkMollie%26MGUID%3D55E96EA9-18CF-8288-F73C-C0A8380A4379%26ObjectPath%3D%2FShops%2FDemoShop&method=&metadata=HASH(0x1418ca78)&webhookUrl=http%3A%2F%2Fmy-vbox-10.intern.epages.de%3A8088%2Frs%2Fpayments%2Fmollie%2F17739%2Fupdate&locale=en_GB&apikey=test_2MU4Y7c6tEHb5igsFd6HWs74aaxvJH',
        $aParameterTypesGetReturn
    );

    my $aParameterTypesTokenNumber = ['object'];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_getTokenNumber',
        'kjhfewdkihuewf9823895409oi', $aParameterTypesTokenNumber );

    my $aParameterTypesPaymentRequest = [ 'object', 'string' ];

    if ( $PaymentRequestOK == 1 ) {
        $Mockify->addMockWithReturnValueAndParameterCheck(
            '_createPaymentRequest',
            $self->_createPaymentRequestOK(),
            $aParameterTypesPaymentRequest
        );
    }
    elsif ( $PaymentRequestOK == 2 ) {
        $Mockify->addMockWithReturnValueAndParameterCheck(
            '_createPaymentRequest',
            $self->_createPaymentRequestEmpty(),
            $aParameterTypesPaymentRequest
        );
    }
    else {
        $Mockify->addMockWithReturnValueAndParameterCheck(
            '_createPaymentRequest',
            $self->_createPaymentRequestError(),
            $aParameterTypesPaymentRequest
        );
    }

    return $Mockify->getMockObject();
}

#=======================================================================================
sub _createPaymentRequestOK {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify        = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Mollie::API::PaymentRequest',
        $aParameterList );

    my $aParameterTypescreateRequest = ['hashref','string'];
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'createRequest',
        {
            'id' => 'tr_MU84QbtAms',
            'paymentUrl' =>
'https://www.mollie.com/paymentscreen/creditcard/testmode/dc0aa798840b6cfacc203277df02174f'
        },
        $aParameterTypescreateRequest
    );

    return $Mockify->getMockObject();
}

#=======================================================================================
sub _createPaymentRequestEmpty {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify        = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Mollie::API::PaymentRequest',
        $aParameterList );

    my $aParameterTypescreateRequest = ['hashref','string'];
    $Mockify->addMockWithReturnValueAndParameterCheck( 'createRequest',
        { 'id' => '', 'paymentUrl' => '' },
        $aParameterTypescreateRequest );

    return $Mockify->getMockObject();
}

#=======================================================================================
sub _createPaymentRequestError {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify        = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Mollie::API::PaymentRequest',
        $aParameterList );

    $Mockify->addMock(
        'createRequest',
        sub {
            shift;
            Error('Error Connecting epagesj');
            return;
        }
    );

    return $Mockify->getMockObject();
}

sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'Reseller' => 'epages_default'
    };
    my $PaymentMethod = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Mollie::API::Object::PaymentMethodMollie', $hAttributes );

    return $PaymentMethod;
}

__PACKAGE__->runScript;
