use Test::More tests => 8;
use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Order::API::Order qw ( CreateOrderWithContainer );


sub TestOnBeforeUpdateShop {

    # Create an order in the test shop
    my $System = LoadRootObject();
    my $Shop = $System->child('TestShop');
    my $CustomerNo = GetNextCustomerNo($Shop);
    my $CustomerFolder = GetCustomerFolder($Shop);
    my $Customer = $Shop->get('CustomerClass')->insertObject({
        Parent => $CustomerFolder,
        Alias  => $CustomerNo,
    });
    my $Order = CreateOrderWithContainer($Customer);

    # Find the default (system) TaxMatrix for Italy and Germany
    # to replace the German default shop TaxMatrix with the Italian one, then in
    # the last step replace the Italian one with the German TaxMatrix again.
    my $aTaxMatrixes = LoadRootObject()->childrenByClass('TaxMatrix');
    my $ItalianTaxMatrix = undef;
    my $GermanTaxMatrix = undef;
    foreach my $TaxMatrix (@$aTaxMatrixes) {
        my $Alias = $TaxMatrix->alias;
        $ItalianTaxMatrix = $TaxMatrix if $Alias eq 'TaxMatrixItaly';
        $GermanTaxMatrix = $TaxMatrix if $Alias eq 'TaxMatrixGermany';
    }
    ok(defined $ItalianTaxMatrix, 'Default Italian TaxMatrix found');
    ok(defined $GermanTaxMatrix, 'Default German TaxMatrix found');

    # Init the shop with the German TaxMatrix
    $Shop->set({TaxMatrix => $GermanTaxMatrix});

    # Test the hook with just a shop, order and empty values
    my $hParams = {
        Object => $Shop,
        Values => (),
    };
    eval {
        DE_EPAGES::Order::Hooks::Shop::OnBeforeUpdateShop($hParams);
    };
    ok(!ExistsError(), 'No error in OnBeforeUpdateShop with empty values');

    # Test the hook with just having the identical TaxMatrix
    $hParams = {
        Object => $Shop,
        Values => {
            TaxMatrix => $GermanTaxMatrix,
        },
    };
    eval {
        DE_EPAGES::Order::Hooks::Shop::OnBeforeUpdateShop($hParams);
    };
    ok(!ExistsError(), 'No error in OnBeforeUpdateShop with empty TaxMatrix');

    # Test the hook with a normal TaxMatrix
    $hParams = {
        Object => $Shop,
        Values => {
            'TaxMatrix' => $ItalianTaxMatrix,
        },
    };
    eval {
        DE_EPAGES::Order::Hooks::Shop::OnBeforeUpdateShop($hParams);
    };
    ok(!ExistsError(), 'No error in OnBeforeUpdateShop with empty TaxMatrix');

    # Delete the LineItemContainer of the order
    my $LineItemContainer = $Order->get('LineItemContainer');
    $Order->set({ LineItemContainer => undef });
    $LineItemContainer->delete;

    # Now test the hook with the broken order and a normal TaxMatrix
    $hParams = {
        Object => $Shop,
        Values => {
            TaxMatrix => $ItalianTaxMatrix,
        },
    };
    eval {
        DE_EPAGES::Order::Hooks::Shop::OnBeforeUpdateShop($hParams);
    };
    ok(!ExistsError(), 'No error in OnBeforeUpdateShop with broken order and a new TaxMatrix');

    return;
}

sub Cleanup {
    my $System = LoadRootObject();
    if( $System->existsChild('TestShop') ) {
        $System->child('TestShop')->delete;
    }

    return 1;
}

sub Init {
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild('Shop', { Alias => 'TestShop' } );

    return 1;
}

RunScript( Message => 'long', Sub => sub {
    RunOnStore(
        Store => GetTestSite(),
        Sub   => sub {
            ok( Init(), 'Initialization' );

            TestOnBeforeUpdateShop();

            ok( Cleanup(), 'Cleanup' );
        }
    );
});

1;
