#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::PatchPayment_RequestRepository
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for PatchPayment::RequestRepository
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::PatchPayment_RequestRepository;
use base DE_EPAGES::Core::API::Test;


use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestDTO;
use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestRepository;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_collectData_WithBillingAddressOnly();
    $self->test_collectData_WithBillingAndShippingAddress();

    done_testing();
}

sub test_collectData_WithBillingAddressOnly {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # prepare mocks
    my $BillingAddress = $self->_buildAddress({
        'FullName' =>  'Dagobert Duck',
        'Street' =>  'Dagobertstr. 1',
        'Zipcode' =>  '77777',
        'City' =>  'Entenhausen',
        'State' =>  'CA',
        'CountryID' =>  'DE',
    });

    my $Basket = $self->_buildBasket({
        'BillingAddress' => $BillingAddress,
        'ShippingAddress' => undef,
    });

    my $BillingAddressDTO = $self->_buildAddressDTO($BillingAddress);
    my $BillingAddressRepository = $self->_buildAddressRepository($BillingAddressDTO);

    my $ShippingAddressDTO = $self->_buildAddressDTO($BillingAddress);
    my $ShippingAddressRepository = $self->_buildAddressRepository($ShippingAddressDTO);

    my $SUT = $self->_createRequestRepositiory();

    # inject mocks
    $SUT->setBillingAddressRepository($BillingAddressRepository);
    $SUT->setShippingAddressRepository($ShippingAddressRepository);

    # test
    ok($SUT->collectData($Basket), "$SubTestName - test if the collect is successful");

    my $RequestDTO = $SUT->getRequestDTO();

    my $hExpectedPatchPaymentRequest = {
          'basketCurrency' => 'EUR',
          'subTotalPrice' => '61.30',
          'basketPrice' => '74.95',
          'taxPrice' => '10.00',
          'shippingPrice' => '8.65',
          'discountPrice' => '-10',
          'handlingFeePrice' => '5',
          'paymentId' => 'DummyPaymentId',
          'shippingAddress' => {
             'city' => 'Entenhausen',
             'state' =>  'CA',
             'recipientName' => 'Dagobert Duck',
             'line1' => 'Dagobertstr. 1',
             'countryCode' => 'DE',
             'postalCode' => '77777',
          },
          'billingAddress' => {
             'city' => 'Entenhausen',
             'state' =>  'CA',
             'recipientName' => 'Dagobert Duck',
             'line1' => 'Dagobertstr. 1',
             'countryCode' => 'DE',
             'postalCode' => '77777',
          },
    };
    is_deeply($RequestDTO->toHash() , $hExpectedPatchPaymentRequest, "$SubTestName - test if hash was set correctly");
}
sub test_collectData_WithBillingAndShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # prepare mocks
    my $BillingAddress = $self->_buildAddress({
        'FullName' =>  'Dagobert Duck',
        'Street' =>  'Dagobertstr. 1',
        'Zipcode' =>  '77777',
        'City' =>  'Entenhausen',
        'State' =>  'CA',
        'CountryID' =>  'DE',
    });

    my $ShippingAddress = $self->_buildAddress({
        'FullName' =>  'Donald Duck',
        'Street' =>  'Meanstreet. 22A',
        'Zipcode' =>  '11111',
        'City' =>  'Dogtown',
        'State' =>  'CA',
        'CountryID' =>  'DE',
    });

    my $Basket = $self->_buildBasket({
        'BillingAddress' => $BillingAddress,
        'ShippingAddress' => $ShippingAddress,
    });

    my $BillingAddressDTO = $self->_buildAddressDTO($BillingAddress);
    my $BillingAddressRepository = $self->_buildAddressRepository($BillingAddressDTO);

    my $ShippingAddressDTO = $self->_buildAddressDTO($ShippingAddress);
    my $ShippingAddressRepository = $self->_buildAddressRepository($ShippingAddressDTO);

    my $SUT = $self->_createRequestRepositiory();

    # inject mocks
    $SUT->setBillingAddressRepository($BillingAddressRepository);
    $SUT->setShippingAddressRepository($ShippingAddressRepository);

    # test
    ok($SUT->collectData($Basket), "$SubTestName - test if the collect is successful");

    my $RequestDTO = $SUT->getRequestDTO();

    my $hExpectedPatchPaymentRequest = {
          'basketCurrency' => 'EUR',
          'subTotalPrice' => '61.30',
          'basketPrice' => '74.95',
          'taxPrice' => '10.00',
          'shippingPrice' => '8.65',
          'discountPrice' => '-10',
          'handlingFeePrice' => '5',
          'paymentId' => 'DummyPaymentId',
          'shippingAddress' => {
             'city' => 'Dogtown',
             'state' =>  'CA',
             'recipientName' => 'Donald Duck',
             'line1' => 'Meanstreet. 22A',
             'countryCode' => 'DE',
             'postalCode' => '11111',
          },
          'billingAddress' => {
             'city' => 'Entenhausen',
             'state' =>  'CA',
             'recipientName' => 'Dagobert Duck',
             'line1' => 'Dagobertstr. 1',
             'countryCode' => 'DE',
             'postalCode' => '77777',
          },
    };
    is_deeply($RequestDTO->toHash() , $hExpectedPatchPaymentRequest, "$SubTestName - test if hash was set correctly");
}
#----------------------------------------------------------------------------------------
#   Helpers
#----------------------------------------------------------------------------------------
sub _buildAddressDTO {
    my $self = shift;
    my ($Address) = @_;

    my $AddressDTO = DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::AddressDTO->new();
    $AddressDTO->recipientName($Address->get('FullName'));
    $AddressDTO->line1($Address->get('Street'));
    $AddressDTO->postalCode($Address->get('Zipcode'));
    $AddressDTO->city($Address->get('City'));
    $AddressDTO->countryState($Address->get('State'));
    $AddressDTO->countryCode($Address->get('CountryID'));

    return $AddressDTO;
}
#----------------------------------------------------------------------------------------
sub _buildAddressRepository {
    my $self = shift;
    my ($AddressDTO) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::AddressRepository', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('collectData', 1, [{'object' => 'DE_EPAGES::Address::API::Object::Address'}] );
    $Mockify->addMockWithReturnValue('getAddressDTO', $AddressDTO);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRequestRepositiory {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::RequestRepository', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('_createPaymentCollectData', 1, [{'object' => 'DE_EPAGES::Order::API::Object::Basket'}] );
    $Mockify->addMockWithReturnValue('_getCreatePaymentRequestDTO', $self->_createPaymentRequestDTO());

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentRequestDTO {
    my $self = shift;

    my $CreatePaymentRequestDTO = DE_EPAGES::PayPalPlus::API::Webservice::CreatePayment::DTO::Request->new();
    $CreatePaymentRequestDTO->basketPrice('74.95');
    $CreatePaymentRequestDTO->subTotalPrice('61.30');
    $CreatePaymentRequestDTO->shippingPrice('8.65');
    $CreatePaymentRequestDTO->taxPrice('10.00');
    $CreatePaymentRequestDTO->discountPrice('-10');
    $CreatePaymentRequestDTO->handlingFeePrice('5');
    $CreatePaymentRequestDTO->basketCurrency('EUR');

    return $CreatePaymentRequestDTO;
}
#----------------------------------------------------------------------------------------
sub _buildBasket {
    my $self = shift;
    my ($hAdditionalAttributes) =  @_;

    my $hAttributes = {
        'Alias' => 'Basket',
        'LineItemContainer' => $self->_createLineItemContainer(),
        %$hAdditionalAttributes,
    };
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );
    return $Basket;
}
#----------------------------------------------------------------------------------------
sub _buildAddress {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Address = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
    return $Address;
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment' => $self->_createLineItemPayment()
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
    return $LineItemContainer;
}

#----------------------------------------------------------------------------------------
sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = {
        'TransID2' => 'DummyPaymentId',
    };
    my $LineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus', $hAttributes );
    return $LineItemPayment;
}
__PACKAGE__->runScript();